﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;

namespace E39
{
    /// <summary>
    /// Config class which loads the configuration from an app.config file.
    /// </summary>
    public class ConfigHandler : Config
    {
        public ConfigHandler()
        {
            DownloadUrl = ConfigurationManager.AppSettings["DownloadUrl"] ?? "";
            string FileDataTypes_string = ConfigurationManager.AppSettings["FileDataTypes"];
            string AllStationNames_string = ConfigurationManager.AppSettings["AllStationNames"];
            PythonExecutablePath = ConfigurationManager.AppSettings["PythonExecutablePath"] ?? "python";
            RawDataDirectory = ConfigurationManager.AppSettings["RawDataDirectory"] ?? Environment.CurrentDirectory;
            ExcelOutputDirectory = ConfigurationManager.AppSettings["ExcelOutputDirectory"] ?? Environment.CurrentDirectory;
            ExcelTemplatesDirectory = ConfigurationManager.AppSettings["ExcelTemplatesDirectory"] ?? Environment.CurrentDirectory;
            string ExcelStartRow_string = ConfigurationManager.AppSettings["ExcelStartRow"];
            string ExcelStartColumn_string = ConfigurationManager.AppSettings["ExcelStartColumn"];
            string ExcelFeaturesOrder_string = ConfigurationManager.AppSettings["ExcelFeaturesOrder"];
            string ExcelScatFeaturesOrder_string = ConfigurationManager.AppSettings["ExcelScatFeaturesOrder"];
            string FeaturesToImpute_string = ConfigurationManager.AppSettings["FeaturesToImpute"];
            string KNNParameter_K_string = ConfigurationManager.AppSettings["KNNParameter_K"];
            string KNNParameter_N_string = ConfigurationManager.AppSettings["KNNParameter_N"];

            FileDataTypes = FileDataTypes_string?.Split(';').ToList() ?? new List<string>
            {
                "wind",
                "wave"
            };
            AllStationNames = AllStationNames_string?.Split(';').ToList() ?? new List<string>
            {
                "A_Sulafjorden",
                "B_Sulafjorden",
                "C_Sulafjorden",
                "D_Breisundet",
                "F_Vartalsfjorden",
                "G_Halsafjorden",
                "G1_Halsafjorden",
                "G2_Halsafjorden"
            };

            if (ExcelStartRow_string == null) { ExcelStartRow = 1; }
            else { ExcelStartRow = int.Parse(ExcelStartRow_string); }

            if (ExcelStartColumn_string == null) { ExcelStartColumn = 1; }
            else { ExcelStartColumn = int.Parse(ExcelStartColumn_string); }

            if (ExcelFeaturesOrder_string == null)
            {
                ExcelFeaturesOrder = new List<string>
                {
                    "timeInterval", "time", "Hm0", "hm0a", "hm0b", "hmax", "latitude", "longitude",
                    "mdir", "mdira", "mdirb", "sprtp", "thhf", "thmax", "thtp", "tm01", "tm02", "tm02a",
                    "tm02b", "tp", "WindDirection", "WindGust", "WindSpeed"
                };
            }
            else
            {
                ExcelFeaturesOrder = ExcelFeaturesOrder_string.Split(';').ToList();
            }

            if (ExcelScatFeaturesOrder_string == null)
            {
                ExcelScatFeaturesOrder = new List<string>
                {
                    "timeInterval", "Hm0", "hm0a", "hm0b", "thtp", "tp", "WindDirection", "WindSpeed"
                };
            }
            else
            {
                ExcelScatFeaturesOrder = ExcelScatFeaturesOrder_string.Split(';').ToList();
            }

            if (FeaturesToImpute_string == null)
            {
                FeaturesToImpute = new List<string> {
                    "Hm0", "hm0a", "hm0b", "hmax", "latitude", "longitude", "mdir", "mdira", "mdirb",
                    "sprtp", "thhf", "thmax", "thtp", "tm01", "tm02", "tm02a", "tm02b", "tp",
                    "WindSpeed", "WindGust", "WindDirection"
                };
            }
            else
            {
                FeaturesToImpute = FeaturesToImpute_string.Split(';').ToList<string>();
            }

            int K = int.TryParse(KNNParameter_K_string, out K) ? K : 5;   // Try and parse K, if not use default 5
            int N = int.TryParse(KNNParameter_N_string, out N) ? N : 100; // Try and parse N, if not use default 100
            KNNParameter_K = K;
            KNNParameter_N = N;

            bool debug = bool.TryParse(ConfigurationManager.AppSettings["Debug"], out debug) ? debug : false;
            Debug = debug;
        }
    }
}