﻿using System.Collections.Generic;
using System.Data;
using System.Linq;

namespace E39.Imputation
{
    /// <summary>
    /// Represents a single empty data point, with its neighbor data points and their respective weights and distances from the emtpy data point.
    /// </summary>
    public class KNNDataPointNeighbors
    {
        /// <summary>
        /// Represents a neighboring data point, with its weight and distance from the empty data point.
        /// </summary>
        public class KNNDistance
        {
            public DataRow DataPointTo { get; set; }
            public double Distance { get; set; }
            public double Weight { get; set; }
            public double? Coefficient { get; set; }
            public double? SpatialDist { get; set; }
            public double? TimeDist { get; set; }

            public KNNDistance(DataRow dataPointTo, double distance, double weight)
            {
                DataPointTo = dataPointTo;
                Distance = distance;
                Weight = weight;
            }
        }

        public DataRow DataPoint { get; set; }
        public List<KNNDistance> Neighbors { get; set; }
        public double? WeightSum { get; set; }

        public KNNDataPointNeighbors(DataRow dataPoint)
        {
            DataPoint = dataPoint;
            Neighbors = new List<KNNDistance>();
        }

        public KNNDistance AddNeighbor(DataRow dataPointTo, double distance, double weight)
        {
            var dist = new KNNDistance(dataPointTo, distance, weight);
            Neighbors.Add(dist);
            return dist;
        }

        public override string ToString()
        {
            string result = string.Format("KNNDataPointNeighbors: from DataPoint: {0} {1}, WeightSum: {2}, CoefficientSum: {3} {{\n",
                DataPoint.Table.ExtendedProperties["StationName"],
                DataPoint["timeInterval"],
                WeightSum.HasValue ? WeightSum.ToString() : "not set",
                Neighbors.Sum(n => n.Coefficient));

            foreach (var neighbor in Neighbors)
            {
                result += string.Format("    to DataPoint: {0} {1}, Distance: {2}, Weight: {3}, Coefficient: {4}, SpatialDist: {5}, TimeDist: {6}\n",
                    neighbor.DataPointTo.Table.ExtendedProperties["StationName"],
                    neighbor.DataPointTo["timeInterval"],
                    neighbor.Distance,
                    neighbor.Weight,
                    neighbor.Coefficient.HasValue ? neighbor.Coefficient.ToString() : "not set",
                    neighbor.SpatialDist.HasValue ? neighbor.SpatialDist.ToString() : "not set",
                    neighbor.TimeDist.HasValue ? neighbor.TimeDist.ToString() : "not set"
                );
            }

            return result + "}";
        }
    }
}