﻿using System;
using System.Data;

namespace E39.Imputation
{
    /// <summary>
    /// Class for doing common final processing steps after imputing missing data points.
    /// </summary>
    public class PostProcessor
    {
        public DataSet StationsData { get; }

        public PostProcessor(DataSet stationsData)
        {
            StationsData = stationsData;
        }

        /// <summary>
        /// Copies the timeInterval field for all imputed data points into "time" feature field.
        /// </summary>
        /// <remarks>
        /// Since "time" feature is of data type DateTime, imputation algorithms ordinarily can't impute it.
        /// This method is therefore for copying the timeInterval field into the time field as a final step
        /// after imputing all other features.
        /// </remarks>
        public void CopyTimeFeatureFromTimeInterval()
        {
            foreach (DataTable station in StationsData.Tables)
            {
                foreach (var dataPoint in station.Select("Imputed = true"))
                {
                    if (dataPoint.IsNull("time") || new DateTimeOffset(dataPoint.Field<DateTime>("time"), TimeSpan.Zero).ToUnixTimeSeconds() == 0)
                    {
                        dataPoint["time"] = dataPoint.Field<DateTime>("timeInterval");
                    }
                }
            }
        }
    }
}