import csv
from netCDF4 import Dataset
from itertools import zip_longest


def convert_vars_in_root(nc_file_path, csv_file_path):
    """
    Converts all variables in the root group of a NetCDF file to a csv file.
    :param nc_file_path: the path of the NetCDF input file
    :param csv_file_path: the path of the csv output file. The file is created if it doesn't exist
    """

    # Root group of the NetCDF file. All measurement data exists in this group
    nc_root = Dataset(nc_file_path)

    with open(csv_file_path, mode='w+', newline='') as csv_file:
        csv_writer = csv.writer(csv_file, dialect='excel', delimiter=';')

        # Write header to csv file, which consist of the variable names from the root groups, aka. the names of the features in the data set
        variable_names = [name for name in nc_root.variables]
        csv_writer.writerow(variable_names)

        # Write rows with measurement data
        variables = [var for var in nc_root.variables.values()]
        csv_writer.writerows(zip_longest(*variables))


if __name__ == '__main__':
    import sys

    if len(sys.argv) < 3:
        print('Usage: python NetCDFConverter.py <input netcdf file path> <output csv file path>')
    else:
        nc_file_path = sys.argv[1]
        csv_file_path = sys.argv[2]
        try:
            convert_vars_in_root(nc_file_path, csv_file_path)
        except FileNotFoundError as ex:
            exit(99)  # Exit code 99 means the input NetCDF file doesn't exist
