﻿using NUnit.Framework;
using System.Data;

namespace E39_Tests
{
    /// <summary>
    /// Helper methods for doing Asserts in unit tests on different data types.
    /// </summary>
    public class AssertHelpers
    {
        /// <summary>
        /// Asserts that two DataTables are equal, both in column names and order, and row content and order.
        /// </summary>
        /// <param name="expectedDt">The expected data table structure and content.</param>
        /// <param name="actualDt">The actual data table.</param>
        public static void AssertDataTablesAreEqual(DataTable expectedDt, DataTable actualDt)
        {
            Assert.NotNull(expectedDt);
            Assert.NotNull(actualDt);

            Assert.AreEqual(expectedDt.Rows.Count, actualDt.Rows.Count, "Number of rows differ");
            Assert.AreEqual(expectedDt.Columns.Count, actualDt.Columns.Count, "Number of columns differ");

            for (int col = 0; col < expectedDt.Columns.Count; ++col)
            {
                Assert.AreEqual(expectedDt.Columns[col].ColumnName, actualDt.Columns[col].ColumnName, "Column names or order differ");
            }

            for (int row = 0; row < expectedDt.Rows.Count; ++row)
            {
                Assert.AreEqual(expectedDt.Rows[row].ItemArray, actualDt.Rows[row].ItemArray, $"Row elements differ at row index {row}");
            }
        }
    }
}
