using System;
using NUnit.Framework;
using E39;
using E39.FileDownloading;

namespace E39_Tests.E39_Lib.FileDownloading
{
    public class FileGetterTests
    {
        [SetUp]
        public void Setup()
        {
        }

        [Test]
        public void DownloadFile_Throws_ArgumentException_If_url_Parameter_Has_Wrong_Format()
        {
            // Arrange
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);
            var url = "/wrongformat/";

            // Act
            var e = Assert.Catch(() => fileGetter.DownloadFile(url, "C:/Users/trygje/Desktop"));

            // Asserts
            Assert.IsInstanceOf<ArgumentException>(e);
        }

        [Test]
        public void DownloadFile_Returns_null_If_File_Is_Not_Successfully_Downloaded()
        {
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);
            var destinationURI = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);

            var returnBoolean = fileGetter.DownloadFile("http://google.com/filename", destinationURI);

            Assert.IsFalse(returnBoolean);
        }

        [Test]
        public void DownloadFile_Adds_Insuccesfull_URL_To_FailedDownloadURLs()
        {
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);
            var destinationURI = Environment.GetFolderPath(Environment.SpecialFolder.Desktop);
            var url = "http://google.com/filename";
            fileGetter.DownloadFile(url, destinationURI);

            var actualURL = fileGetter.FailedDownloadURLs[0];

            Assert.AreEqual(url, actualURL);
        }

        [Test]
        public void GetFilename_Returns_Filename()
        {
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);
            var url = "http://google.com/filename";

            var actual = FileGetter.GetFilename(url);

            Assert.AreEqual("filename", actual);
        }
        
        [Test]
        public void GetFilename_Returns_Null_For_Invalid_URL()
        {
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);
            var url = "";

            var actual = FileGetter.GetFilename(url);

            Assert.IsNull(actual);
        }

        [Test]
        public void DownloadFiles_Throws_ArgumentException_If_Date_Parameters_Are_Invalid()
        {
            var ch = new ConfigHandler();
            var fileGetter = new FileGetter(ch);

            var e = Assert.Catch(() => fileGetter.DownloadFiles(2019, 2018, 1, 1));  // startYear, endYear, startMonth, endMonth
            var e1 = Assert.Catch(() => fileGetter.DownloadFiles(2018, 2018, 0, 1));  // startYear, endYear, startMonth, endMonth
            var e2 = Assert.Catch(() => fileGetter.DownloadFiles(2018, 2018, 1, 0));  // startYear, endYear, startMonth, endMonth
            var e3 = Assert.Catch(() => fileGetter.DownloadFiles(2018, 2018, 13, 1));  // startYear, endYear, startMonth, endMonth
            var e4 = Assert.Catch(() => fileGetter.DownloadFiles(2018, 2018, 1, 13));  // startYear, endYear, startMonth, endMonth
            var e5 = Assert.Catch(() => fileGetter.DownloadFiles(1969, 2018, 1, 1));  // startYear, endYear, startMonth, endMonth

            Assert.IsInstanceOf<ArgumentException>(e);
            Assert.IsInstanceOf<ArgumentException>(e1);
            Assert.IsInstanceOf<ArgumentException>(e2);
            Assert.IsInstanceOf<ArgumentException>(e3);
            Assert.IsInstanceOf<ArgumentException>(e4);
            Assert.IsInstanceOf<ArgumentException>(e5);
        }
    }
}