﻿using E39.Imputation;
using NUnit.Framework;
using System;
using System.Collections.Generic;
using System.Data;
using System.Text;

namespace E39_Tests.E39_Lib.Imputation
{
    public class DataPointValidatorTests
    {
        private DataRow dp1;

        [SetUp]
        public void SetUp_CreateDataTableStructure()
        {
            var dt = new DataTable();
            dt.Columns.Add("timeInterval", typeof(DateTime));
            dt.Columns.Add("time", typeof(DateTime));
            dt.Columns.Add("latitude", typeof(double));
            dt.Columns.Add("longitude", typeof(double));
            dt.Columns.Add("Hm0", typeof(double));
            dt.Columns.Add("thtp", typeof(double));
            dt.Columns.Add("tp", typeof(double));

            dp1 = dt.NewRow();
            dp1["time"] = new DateTime(2019, 1, 1, 0, 0, 0, DateTimeKind.Utc);
        }

        [Test]
        public void IsMissingCoordinates_DBNullValues_ReturnsIsMissing()
        {
            dp1["latitude"] = DBNull.Value;
            dp1["longitude"] = DBNull.Value;

            Assert.IsTrue(DataPointValidator.IsMissingCoordinates(dp1));
        }

        [Test]
        public void IsDataPointEmpty_DatatypeNotDouble_ReturnsIsEmpty()
        {
            var dt = new DataTable();
            dt.Columns.Add("time", typeof(DateTime));
            dt.Columns.Add("Hm0", typeof(string));
            dt.Columns.Add("thtp", typeof(double));
            dt.Columns.Add("tp", typeof(double));

            dp1 = dt.NewRow();
            dp1["time"] = new DateTime(2019, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            dp1["Hm0"] = "1.0";  // Not able to cast to double should be treated like zero
            dp1["thtp"] = 0.0;
            dp1["tp"] = 0.0;

            Assert.IsTrue(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_DBNullValues_ReturnsIsEmpty()
        {
            Assert.IsTrue(dp1["Hm0"].Equals(DBNull.Value));
            Assert.IsTrue(dp1["thtp"].Equals(DBNull.Value));
            Assert.IsTrue(dp1["tp"].Equals(DBNull.Value));
            Assert.IsTrue(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_NaNValues_ReturnsIsEmpty()
        {
            dp1["Hm0"] = double.NaN;
            dp1["thtp"] = double.NaN;
            dp1["tp"] = double.NaN;
            Assert.IsTrue(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_ZeroValues_ReturnsIsEmpty()
        {
            dp1["Hm0"] = 0.0;
            dp1["thtp"] = 0.0;
            dp1["tp"] = 0.0;
            Assert.IsTrue(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_TimeIsZeroUnixTimestamp_ReturnsIsEmpty()
        {
            dp1["time"] = new DateTime(1970, 1, 1, 0, 0, 0, DateTimeKind.Utc);
            Assert.IsTrue(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_OneNonzeroValue_ReturnsIsNotEmpty()
        {
            dp1["Hm0"] = 1.0;
            dp1["thtp"] = double.NaN;
            dp1["tp"] = DBNull.Value;
            Assert.IsFalse(DataPointValidator.IsDataPointEmpty(dp1));
        }

        [Test]
        public void IsDataPointEmpty_OneEpsilonValue_ReturnsIsNotEmpty()
        {
            dp1["tp"] = double.Epsilon;
            Assert.IsFalse(DataPointValidator.IsDataPointEmpty(dp1));
        }
    }
}
