﻿using System;
using System.Collections.Generic;
using NUnit.Framework;
using E39.WebApplication.Scheduler;
using System.IO;
using E39.WebApplication;

namespace E39_Tests.E39_WebApplication.Scheduler
{
    class FileCacherTests
    {

        private WebAppConfig config;

        [SetUp]
        public void Setup()
        {
            config = new WebAppConfig
            {
                AllStationNames = new List<string> { "E39_A_Sulafjorden" },
                FileDataTypes = new List<string> { "wind", "wave" },
                CacherFileInfoUrl = "http://thredds.met.no/thredds/dodsC/obs/buoy-svv-e39",
                DownloadUrl = "http://thredds.met.no/thredds/fileServer/obs/buoy-svv-e39",
                RawDataDirectory = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), "CACHED_FILES")
            };
        }

        [Test]
        public void GetDateModified_Returns_The_Correct_Date()
        {
            // Arrange
            var url = "http://thredds.met.no/thredds/dodsC/obs/buoy-svv-e39/2016/10/201610_E39_A_Sulafjorden_raw_wind.nc.das";
            var expectedDate = DateTime.Parse("2018-01-19T13:49:22Z");

            // Act
            var actualDate = FileCacher.GetDateModified(url);

            // Asserts
            Assert.That(expectedDate, Is.EqualTo(actualDate));
        }

        [Test]
        public void GetDateModified_Returns_Null_When_URL_Is_Incorrect()
        {
            // Arrange
            var url = "http://thredds.met.no/thredds/rævmat";
            var expectedDate = DateTime.Parse("2018-01-19T13:49:22Z");

            // Act
            var returnvalue = FileCacher.GetDateModified(url);

            // Asserts
            Assert.IsNull(returnvalue);
        }

        [Test]
        public void GetDateModified_Returns_A_DateTime_Object_When_URL_Is_Correct()
        {
            // Arrange
            var url = "http://thredds.met.no/thredds/dodsC/obs/buoy-svv-e39/2016/12/201612_E39_G_Halsafjorden_wind.nc.das";

            // Act
            var dateTime = FileCacher.GetDateModified(url);

            // Asserts
            Assert.IsInstanceOf(typeof(DateTime), dateTime);
        }

        //TODO: Fix this test
        [Test]
        public void CheckForModifiedFiles_Checks_For_Modified_Files()
        {
            // Arrange
            var filecacher = new FileCacher(config, null, null);
            var startDate = new DateTime(2017, 11, 1, 0, 0, 0);
            var endDate = new DateTime(2017, 12, 1, 0, 0, 0);

            // Act
            filecacher.CheckForModifiedFilesAsync(startDate, endDate, config.RawDataDirectory);

            // Asserts
            Assert.True(true);
        }

        [Test]
        public void CheckForModifiedFiles_Throws_ArgurmentException_When_Parameter_startDate_Is_After_endDate()
        {
            var filecacher = new FileCacher(config, null, null);
            var startDate = new DateTime(2017, 11, 1, 0, 0, 0);
            var endDate = new DateTime(2016, 12, 1, 0, 0, 0);

            var e = Assert.Catch(() => filecacher.CheckForModifiedFilesAsync(startDate, endDate, "Uri"));

            Assert.IsInstanceOf<ArgumentException>(e);
        }
    }
}