﻿using E39.WebApplication.Models;
using Microsoft.EntityFrameworkCore;
using System.Threading.Tasks;

namespace E39.WebApplication.Data
{
    public class FileDbContext : DbContext
    {
        public FileDbContext(DbContextOptions<FileDbContext> options)
            : base(options)
        {
        }

        public DbSet<Report> Reports { get; set; }
        public DbSet<RawDataFile> RawDataFiles { get; set; }
        public DbSet<ReportRawDataFile> ReportRawDataFiles { get; set; }
        public DbSet<LoadTimes> LoadTimes { get; set; }

        protected override void OnModelCreating(ModelBuilder modelBuilder)
        {
            modelBuilder.Entity<RawDataFile>()
                .Property(rawDataFile => rawDataFile.ManuallyUploaded)
                .HasDefaultValue(false);

            modelBuilder.Entity<ReportRawDataFile>()
                .HasKey(reportRawDataFile => new { reportRawDataFile.ReportFilename, reportRawDataFile.RawDataFileFilename });

            modelBuilder.Entity<ReportRawDataFile>()
                .HasOne(reportRawDataFile => reportRawDataFile.Report)
                .WithMany(report => report.RawDataFilesRelations)
                .HasForeignKey(reportRawDataFile => reportRawDataFile.ReportFilename);

            modelBuilder.Entity<ReportRawDataFile>()
                .HasOne(reportRawDataFile => reportRawDataFile.RawDataFile)
                .WithMany(rawDataFile => rawDataFile.ReportRelations)
                .HasForeignKey(reportRawDataFile => reportRawDataFile.RawDataFileFilename);
        }

        /// <summary>
        /// Loads all related <see cref="Report"/> objects for a <see cref="RawDataFile"/>.
        /// </summary>
        /// <param name="rawDataFile">The <see cref="RawDataFile"/> object for which to load related <see cref="Report"/>s.</param>
        public async Task LoadRelatedAsync(RawDataFile rawDataFile)
        {
            await Entry(rawDataFile)
                .Collection(r => r.ReportRelations)
                .Query()
                .Include(r => r.Report)
                .LoadAsync();
        }

        /// <summary>
        /// Loads all related <see cref="RawDataFile"/> objects for a <see cref="Report"/>.
        /// </summary>
        /// <param name="report">The <see cref="Report"/> object for which to load related <see cref="RawDataFile"/>s.</param>
        public async Task LoadRelatedAsync(Report report)
        {
            await Entry(report)
                .Collection(r => r.RawDataFilesRelations)
                .Query()
                .Include(r => r.RawDataFile)
                .LoadAsync();
        }
    }
}