﻿using E39.WebApplication.Jobs;
using Microsoft.AspNetCore.SignalR;
using System;
using System.Collections.Generic;

namespace E39.WebApplication.Hubs
{
    /// <summary>
    /// SignalR hub for sending state, logs and progress of running <see cref="Job"/>s to browsers.
    /// </summary>
    /// <seealso cref="JobHubEventPropagator"/>
    public class JobHub : Hub<IJobHubClient>
    {
        public class JobInfoDTO
        {
            public JobState State { get; set; }
            public List<JobMessage> Log { get; set; }
            public float Progress { get; set; }
            public DateTimeOffset TimeStarted { get; set; }
            public DateTimeOffset TimeEstimatedFinished { get; set; }
            public JobProperties Properties { get; set; }
        }

        private readonly IJobManager JobManager;

        public JobHub(IJobManager jobManager)
        {
            JobManager = jobManager;
        }

        public void GetAllJobInfo(Guid jobId)
        {
            var job = JobManager.GetJob(jobId);
            if (job != null)
            {
                Clients.Caller.ReceiveAllJobInfo(jobId, new JobInfoDTO
                {
                    State = job.State,
                    Log = job.Log,
                    Progress = job.Progress,
                    TimeStarted = job.TimeStarted,
                    TimeEstimatedFinished = job.EstimatedTimeFinished,
                    Properties = job.Properties
                });
            }
        }
    }
}