﻿using System;

namespace E39.WebApplication.Jobs
{
    public class JobEventArgs : EventArgs { }

    public class JobStateChangedEventArgs : JobEventArgs
    {
        public JobState PreviousState { get; }
        public JobState NewState { get; }

        public JobStateChangedEventArgs(JobState previousState, JobState newState)
        {
            PreviousState = previousState;
            NewState = newState;
        }
    }

    public class JobMessageAddedEventArgs : JobEventArgs
    {
        public JobMessage Message { get; }

        public JobMessageAddedEventArgs(JobMessage message)
        {
            Message = message;
        }
    }

    public class JobProgressChangedEventArgs : JobEventArgs
    {
        public float PreviousProgress { get; }
        public float NewProgress { get; }

        public JobProgressChangedEventArgs(float previousProgress, float newProgress)
        {
            PreviousProgress = previousProgress;
            NewProgress = newProgress;
        }
    }

    public class JobTimeFinishedEstimatedEventArgs : JobEventArgs
    {
        public DateTimeOffset TimeEstimatedFinished { get; }

        public JobTimeFinishedEstimatedEventArgs(DateTimeOffset timeEstimatedFinished)
        {
            TimeEstimatedFinished = timeEstimatedFinished;
        }
    }
}