﻿using E39.WebApplication.Data;
using E39.WebApplication.Scheduler;
using Microsoft.AspNetCore;
using Microsoft.AspNetCore.Hosting;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Quartz;
using System;

namespace E39.WebApplication
{
    public class Program
    {
        public static void Main(string[] args)
        {
            var host = CreateWebHostBuilder(args).Build();

            using (var scope = host.Services.CreateScope())
            {
                var services = scope.ServiceProvider;
                var logger = services.GetRequiredService<ILogger<Program>>();
                var dbContext = services.GetRequiredService<FileDbContext>();
                var config = services.GetRequiredService<IOptions<WebAppConfig>>().Value;

                try
                {
                    FileDbInitializer.Initialize(dbContext);
                }
                catch (Exception ex)
                {
                    logger.LogError(ex, "An error occurred while seeding the database.");
                }

                // Schedule the file cacher and the file cleaner
                var scheduler = services.GetRequiredService<IScheduler>();
                scheduler.ScheduleJob(
                    JobBuilder.Create<DownloadFilesJob>().Build(),
                    TriggerBuilder.Create()
                        .WithSchedule(CronScheduleBuilder.DailyAtHourAndMinute(config.CacherStartHour, config.CacherStartMinute))
                        .StartNow()
                        .Build()
                );
                scheduler.ScheduleJob(
                    JobBuilder.Create<CleanUpFilesJob>().Build(),
                    TriggerBuilder.Create()
                        .WithSchedule(CronScheduleBuilder.DailyAtHourAndMinute(4, 0))
                        //.WithSimpleSchedule(s => s.WithIntervalInSeconds(10).RepeatForever())
                        .StartNow()
                        .Build()
                );
            }

            host.Run();
        }

        public static IWebHostBuilder CreateWebHostBuilder(string[] args) =>
            WebHost.CreateDefaultBuilder(args)
                .ConfigureLogging((webHostBuilderContext, loggingBuilder) =>
                {
                    loggingBuilder.AddLog4Net();
                })
                .UseStartup<Startup>();
    }
}