﻿using E39.WebApplication.Jobs;
using log4net;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Options;
using Quartz;
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace E39.WebApplication.Scheduler
{
    public class CleanUpFilesJob : IJob
    {
        private readonly IConfiguration configuration;
        private readonly IOptionsMonitor<WebAppConfig> configMonitor;
        private readonly ILog log = LogManager.GetLogger(typeof(CleanUpFilesJob));
        private readonly IJobManager jobManager;

        public CleanUpFilesJob(IConfiguration configuration, IOptionsMonitor<WebAppConfig> configMonitor, IJobManager jobManager)
        {
            this.configuration = configuration;
            this.configMonitor = configMonitor;
            this.jobManager = jobManager;
        }

        public Task Execute(IJobExecutionContext context)
        {
            log.Debug("Hello! It is me, the File Cleaner :D And I am going to clean files!");

            var config = configMonitor.CurrentValue;
            var jobsToDelete = new List<Guid>();
            DirectoryInfo baseTempDir = new DirectoryInfo(config.BaseTempDirectory);

            foreach (var job in jobManager.GetJobs())
            {
                // Only remove jobs which have finished running (COMPLETED, FAILED or ABORTED).
                // Remove them when they finished a week ago or before, or remove them immediately if
                // their zip has been downloaded.
                if (job.State != JobState.CREATED && job.State != JobState.RUNNING
                    && (job.ZipDownloaded || job.TimeFinished > DateTimeOffset.UtcNow.AddDays(7)))
                {
                    log.Debug($"Deleting job {job.Id}, that finished {job.TimeFinished}");

                    jobsToDelete.Add(job.Id);
                    DirectoryInfo[] jobTempDirs = baseTempDir.GetDirectories($"{job.Id}*", SearchOption.TopDirectoryOnly);
                    foreach (var dir in jobTempDirs)
                    {
                        dir.Delete(true);
                    }
                }
            }

            foreach (Guid guid in jobsToDelete)
            {
                jobManager.RemoveJob(guid);
            }

            return Task.CompletedTask;
        }
    }
}