﻿using E39.WebApplication.Data;
using log4net;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.Options;
using Quartz;
using System;
using System.IO;
using System.Threading.Tasks;

namespace E39.WebApplication.Scheduler
{
    public class DownloadFilesJob : IJob
    {
        private readonly IConfiguration configuration;
        private readonly IOptionsMonitor<WebAppConfig> configMonitor;
        private readonly FileDbContext FileDbContext;
        private readonly BlobManager BlobManager;
        private readonly ILog log = LogManager.GetLogger(typeof(DownloadFilesJob));

        public DownloadFilesJob(IConfiguration configuration, IOptionsMonitor<WebAppConfig> configMonitor, FileDbContext dbContext, BlobManager blobManager)
        {
            this.configuration = configuration;
            this.configMonitor = configMonitor;
            FileDbContext = dbContext;
            BlobManager = blobManager;
        }

        public async Task Execute(IJobExecutionContext context)
        {
            var config = configMonitor.CurrentValue;

            DateTime startDate = config.CacherStartDownloadDate;
            DateTime endDate = DateTime.UtcNow;

            FileCacher fileCacher = new FileCacher(config, FileDbContext, BlobManager);
            log.Info($"Downloading files from {startDate.ToString("dd-MM-yyyy HH:mm")} to {endDate.ToString("dd-MM-yyyy HH:mm")}");

            string tempDirName = config.CacherTempDirectory;

            Directory.CreateDirectory(tempDirName);
            await fileCacher.CheckForModifiedFilesAsync(startDate, endDate, tempDirName);
            //await fileCacher.CheckForModifiedFilesAsync(startDate, endDate, config.RawDataDirectory);
            Directory.Delete(tempDirName, true);
            log.Info($"Done checking for modified files.");

            await Task.CompletedTask;
        }
    }
}