﻿using System;
using System.Collections.Generic;

namespace E39.WebApplication
{
    public class WebAppConfig : Config
    {
        /// <summary>
        /// Path to a directory used for temporary saving of report files for zipping.
        /// </summary>
        public string ZipFilesDirectory { get; set; }

        /// <summary>
        /// Base path to a directory used for temporary files for jobs.
        /// </summary>
        public string BaseTempDirectory { get; set; }

        /// <summary>
        /// The minute of the <see cref="CacherStartHour"/> on which the cacher should run.
        /// </summary>
        public int CacherStartMinute { get; set; }

        /// <summary>
        /// The hour of day on which the cacher should run.
        /// </summary>
        public int CacherStartHour { get; set; }

        /// <summary>
        /// The first date of the period to automatically download files for.
        /// </summary>
        public DateTime CacherStartDownloadDate { get; set; }

        /// <summary>
        /// What the caching system is called, displayed to the client.
        /// </summary>
        public string CacherUploadedBy { get; set; }

        /// <summary>
        /// The main url for getting file info in the caching system.
        /// </summary>
        public string CacherFileInfoUrl { get; set; }

        /// <summary>
        /// Path to a temporary directory used by the file cacher.
        /// </summary>
        public string CacherTempDirectory { get; set; }

        /// <summary>
        /// Returns a string representation of the <see cref="WebAppConfig"/> object with all its properties and their values.
        /// </summary>
        /// <returns>The string representation of the <see cref="WebAppConfig"/>.</returns>
        public override string ToString()
        {
            var str = "WebAppConfig {\n";

            foreach (var prop in GetType().GetProperties())
            {
                var propValue = prop.GetValue(this);
                if (propValue is System.Collections.IList list)
                {
                    var strList = new List<string>();
                    foreach (var item in list)
                    {
                        strList.Add(item.ToString());
                    }
                    propValue = string.Join(", ", strList);
                }
                str += $"    {prop.Name}: {propValue}\n";
            }

            return str + "}";
        }
    }
}